
From: john stultz <johnstul@us.ibm.com>

This patch catches a corner case in the lost-tick compensation code.

There is a check to see if we overflowed between reads of the two time
sources, however should the high res time source be slightly slower then
what we calibrated, its possible to trigger this code when no ticks have
been lost.

This patch adds an extra check to insure we have seen more then one tick
before we check for this overflow.  This seems to resolve the remaining
"time doubling" issues that I've seen reported.



 25-akpm/arch/i386/kernel/timers/timer_cyclone.c |    2 +-
 25-akpm/arch/i386/kernel/timers/timer_tsc.c     |    2 +-
 2 files changed, 2 insertions(+), 2 deletions(-)

diff -puN arch/i386/kernel/timers/timer_cyclone.c~lost-tick-corner-fix-A0 arch/i386/kernel/timers/timer_cyclone.c
--- 25/arch/i386/kernel/timers/timer_cyclone.c~lost-tick-corner-fix-A0	Mon Jun 30 15:36:51 2003
+++ 25-akpm/arch/i386/kernel/timers/timer_cyclone.c	Mon Jun 30 15:36:51 2003
@@ -88,7 +88,7 @@ static void mark_offset_cyclone(void)
 	 * between cyclone and pit reads (as noted when 
 	 * usec delta is > 90% # of usecs/tick)
 	 */
-	if (abs(delay - delay_at_last_interrupt) > (900000/HZ)) 
+	if (lost && abs(delay - delay_at_last_interrupt) > (900000/HZ)) 
 		jiffies++;
 }
 
diff -puN arch/i386/kernel/timers/timer_tsc.c~lost-tick-corner-fix-A0 arch/i386/kernel/timers/timer_tsc.c
--- 25/arch/i386/kernel/timers/timer_tsc.c~lost-tick-corner-fix-A0	Mon Jun 30 15:36:51 2003
+++ 25-akpm/arch/i386/kernel/timers/timer_tsc.c	Mon Jun 30 15:36:51 2003
@@ -205,7 +205,7 @@ static void mark_offset_tsc(void)
 	 * between tsc and pit reads (as noted when 
 	 * usec delta is > 90% # of usecs/tick)
 	 */
-	if (abs(delay - delay_at_last_interrupt) > (900000/HZ))
+	if (lost && abs(delay - delay_at_last_interrupt) > (900000/HZ))
 		jiffies++;
 }
 

_
